"""JARVIS bot utility commands."""
import logging
from io import BytesIO

from aiofile import AIOFile, LineReader
from dis_snek import MessageContext, Scale, Snake
from dis_snek.models.discord.file import File
from dis_snek.models.snek.command import message_command


class BotutilCog(Scale):
    """JARVIS Bot Utility Cog."""

    def __init__(self, bot: Snake):
        self.bot = bot
        self.logger = logging.getLogger(__name__)

    @message_command(name="tail")
    async def _tail(self, ctx: MessageContext, count: int = 10) -> None:
        if ctx.author.id != self.bot.owner.id:
            return
        lines = []
        async with AIOFile("jarvis.log", "r") as af:
            async for line in LineReader(af):
                lines.append(line)
                if len(lines) == count:
                    lines.pop(0)
        log = "\n".join(lines)
        await ctx.reply(content=f"```\n{log}\n```")

    @message_command(name="log")
    async def _log(self, ctx: MessageContext) -> None:
        if ctx.author.id != self.bot.owner.id:
            return

        async with AIOFile("jarvis.log", "r") as af:
            with BytesIO() as file_bytes:
                async for chunk in af.read(8192):
                    file_bytes.write(chunk)
                log = File(file_bytes, file_name="jarvis.log")
                await ctx.reply(content="Here's the latest log", file=log)


def setup(bot: Snake) -> None:
    """Add BotutilCog to J.A.R.V.I.S."""
    BotutilCog(bot)
